from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3.search import HepMC3 as hmsearch
import random,math


def test_Pythonization_Search():
    print (dir(hmsearch))
    return 0

if __name__ == "__main__":
    result=1
    try:
     result=test_Pythonization_Search()
    except:
     result=1
    sys.exit(result)








# Translated into python from testPolarization.cc
# garren@fnal.gov, Oct. 2010
# andrii.verbytskyi@mpp.mpg.gov, Nov. 2018
#
# In this example we will place the following event into HepMC "by hand"
#
#     name status pdg_id  parent Px       Py    Pz       Energy      Mass
#  1  !p+!    3   2212    0,0    0.000    0.000 7000.000 7000.000    0.938
#  2  !p+!    3   2212    0,0    0.000    0.000-7000.000 7000.000    0.938
#=========================================================================
#  3  !d!     3      1    1,1    0.750   -1.569   32.191   32.238    0.000
#  4  !u~!    3     -2    2,2   -3.047  -19.000  -54.629   57.920    0.000
#  5  !W-!    3    -24    1,2    1.517   -20.68  -20.605   85.925   80.799
#  6  !gamma! 1     22    1,2   -3.813    0.113   -1.833    4.233    0.000
#  7  !d!     1      1    5,5   -2.445   28.816    6.082   29.552    0.010
#  8  !u~!    1     -2    5,5    3.962  -49.498  -26.687   56.373    0.006

from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

import random,math
from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm

def test_Polarization():
    xout1=hm.WriterAscii ("testPolarization1.dat")
    xout2=hm.WriterAscii("testPolarization2.dat")
    xout4=hm.WriterAsciiHepMC2( "testPolarization4.out" )
    xout5=hm.WriterAscii( "testPolarization5.out" )

    # build the graph, which will look like
    #                       p7                   #
    # p1                   /                     #
    #   \v1__p3      p5---v4                     #
    #         \_v3_/       \                     #
    #         /    \        p8                   #
    #    v2__p4     \                            #
    #   /            p6                          #
    # p2                                         #
    #
    # define a flow pattern as  p1 . p3 . p6
    #                       and p2 . p4 . p5
    #

    # First create the event container, with Signal Process 20, event number 1
    #
    evt=hm.GenEvent(hm.Units.GEV,hm.Units.MM)
    evt.set_event_number(1)
    evt.add_attribute("signal_process_id", hm.IntAttribute(20))
    v1=hm.GenVertex()
    evt.add_vertex( v1 )
    p1=hm.GenParticle( hm.FourVector(0,0,7000,7000),2212, 3 )
    evt.add_particle( p1 )
    p1.add_attribute("flow1", hm.IntAttribute(231))
    p1.add_attribute("flow1", hm.IntAttribute(231))
    p1.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p1.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))

    v2=hm.GenVertex()
    evt.add_vertex( v2 )
    p2=hm.GenParticle(  hm.FourVector(0,0,-7000,7000),2212, 3 )
    evt.add_particle( p2 )
    p2.add_attribute("flow1", hm.IntAttribute(243))
    p2.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p2.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v2.add_particle_in( p2 )

    p3=hm.GenParticle( hm.FourVector(.750,-1.569,32.191,32.238),1, 3 )
    evt.add_particle( p3 )
    p3.add_attribute("flow1", hm.IntAttribute(231))
    p3.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p3.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v1.add_particle_out( p3 )
    p4=hm.GenParticle( hm.FourVector(-3.047,-19.,-54.629,57.920),-2, 3 )
    evt.add_particle( p4 )
    p4.add_attribute("flow1", hm.IntAttribute(243))
    p4.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p4.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v2.add_particle_out( p4 )

    v3=hm.GenVertex()
    evt.add_vertex( v3 )
    v3.add_particle_in( p3 )
    v3.add_particle_in( p4 )
    p6=hm.GenParticle(  hm.FourVector(-3.813,0.113,-1.833,4.233 ),22, 1 )
    evt.add_particle( p6 )
    p6.add_attribute("flow1", hm.IntAttribute(231))
    p6.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p6.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v3.add_particle_out( p6 )
    p5=hm.GenParticle( hm.FourVector(1.517,-20.68,-20.605,85.925),-24, 3 )
    evt.add_particle( p5 )
    p5.add_attribute("flow1", hm.IntAttribute(243))
    p5.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p5.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v3.add_particle_out( p5 )
# create v4
    v4=hm.GenVertex(hm.FourVector(0.12,-0.3,0.05,0.004))
    evt.add_vertex( v4 )
    v4.add_particle_in( p5 )
    p7=hm.GenParticle( hm.FourVector(-2.445,28.816,6.082,29.552), 1,1 )
    evt.add_particle( p7 )
    v4.add_particle_out( p7 )
    p8=hm.GenParticle( hm.FourVector(3.962,-49.498,-26.687,56.373), -2,1 )
    evt.add_particle( p8 )
    v4.add_particle_out( p8 )
    
    evt.add_attribute("signal_process_vertex", hm.IntAttribute(v3.id()))
#the event is complete, we now print it out
    hm.Print.content(evt)
    hm.Print.listing(evt,8)
    print (hm.version())
    print (hm.Print.line(v4,True))
    a=0
    print (evt.particles())
    print (len(evt.particles()))
    print (evt.particles()[0])
    for ip in evt.particles():
     print (hm.Print.line(ip,True))
    xout1.write_event(evt)

# write event in old format
    xout4.write_event(evt)
# make a copy and write it
    xout5.write_event(hm.GenEvent(evt))
# try changing polarization
    p2.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p2.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi))
    xout2.write_event(evt)
    xout1.close()
    xout2.close()
    xout4.close()
    xout5.close()
# now clean-up by deleteing all objects from memory
#
# deleting the event deletes all contained vertices, and all particles
# contained in those vertices
    evt.clear()

    assert((COMPARE_ASCII_FILES("testPolarization1.dat","testPolarization5.out")==0) and (COMPARE_ASCII_FILES("testPolarization1.dat","testPolarization2.dat")!=0))
    return 0
if __name__ == "__main__":
    result=1
    try:
     result=test_Polarization()
    except:
     result=1
    sys.exit(result)



from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
import random

def test_Pythonization_GenEvent():
    evt = hm.GenEvent()
    evt.add_particle(hm.GenParticle())
    print (evt.particles)
    #evt.Particles[0].Momentum = (1, 2, 3, 4)
    #evt.Particles[0].Pid = 5
    evt.add_vertex(hm.GenVertex())
    #evt.Vertices[0].Position = (1, 2, 3, 4)
    #assert len(evt.Particles) == 1
    #assert evt.Particles[0].Id == 1
    #assert evt.Particles[0].Pid == 5
    #assert evt.Particles[0].Momentum == (1, 2, 3, 4)
    #assert len(evt.Vertices) == 1
    #assert evt.Vertices[0].Id == -1
    #assert evt.Vertices[0].Position == (1, 2, 3, 4)
    #assert repr(evt) == "GenEvent(momentum_unit=1, length_unit=0, event_number=0, particles=[GenParticle(FourVector(1, 2, 3, 4), status=0, id=1, production_vertex=0, end_vertex=-1)], vertices=[GenVertex(FourVector(1, 2, 3, 4), status=0, id=-1, particles_in=[], particles_out=[])], run_info=None)"
    return 0

if __name__ == "__main__":
    result=1
    try:
     result=test_Pythonization_GenEvent()
    except:
     result=1
    sys.exit(result)








from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

 
from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
from pyHepMC3.rootIO import HepMC3 as hmrootIO
print(dir(hmrootIO))
def test_IO2():
 inputA=hm.ReaderAsciiHepMC2("inputIO2.hepmc")
 if inputA.failed(): sys.exit(1)
 outputA=hmrootIO.WriterRootTree  ("frominputIO2.root")
 if outputA.failed(): sys.exit(2)
 while  not inputA.failed():
        evt=hm.GenEvent()
        inputA.read_event(evt)
        if inputA.failed(): 
            print("End of file reached. Exit.\n")
            break
        outputA.write_event(evt)
        evt.clear()    
 inputA.close()
 outputA.close()
 
 inputB=hmrootIO.ReaderRootTree ("frominputIO2.root")
 if inputB.failed(): sys.exit(3)
 outputB=hm.WriterAsciiHepMC2("fromfrominputIO2.hepmc")
 if outputB.failed(): sys.exit(4)
 while not inputB.failed():
        evt=hm.GenEvent()
        inputB.read_event(evt)
        if  inputB.failed():  
            print ("End of file reached. Exit.\n")
            break
        outputB.write_event(evt)
        evt.clear()    
 inputB.close()
 outputB.close()
 assert(0 == COMPARE_ASCII_FILES("fromfrominputIO2.hepmc","inputIO2.hepmc"))
 return 0
if __name__ == "__main__":
    result=1
    try:
     result=test_IO2()
    except:
     result=1
    sys.exit(result)



from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

import math,random  
from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
def test_Boost():
    evt=hm.GenEvent(hm.Units.MomentumUnit.GEV,hm.Units.LengthUnit.CM)
    evt.set_event_number(1)
    evt.add_attribute("signal_process_id", hm.IntAttribute(20))
#     create vertex 1
    v1=hm.GenVertex()
    evt.add_vertex( v1 )
    p1=hm.GenParticle( hm.FourVector(1.0,1.0,7000,7000),2212, 3 )
    evt.add_particle( p1 )
    p1.add_attribute("flow1", hm.IntAttribute(231))
    p1.add_attribute("flow1", hm.IntAttribute(231))
    p1.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p1.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))

    v2=hm.GenVertex()
    evt.add_vertex( v2 )
    p2=hm.GenParticle( hm.FourVector(1.0,1.0,-7000,7000),2212, 3 )
    evt.add_particle( p2 )
    p2.add_attribute("flow1", hm.IntAttribute(243))
    p2.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p2.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v2.add_particle_in( p2 )
#    
#     create the outgoing particles of v1 and v2
    p3=hm.GenParticle(hm.FourVector(.750,-1.569,32.191,32.238),1, 3 )
    evt.add_particle( p3 )
    p3.add_attribute("flow1", hm.IntAttribute(231))
    p3.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p3.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v1.add_particle_out( p3 )
    p4=hm.GenParticle(hm.FourVector(-3.047,-19.,-54.629,57.920),-2, 3 )
    evt.add_particle( p4 )
    p4.add_attribute("flow1", hm.IntAttribute(243))
    p4.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p4.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v2.add_particle_out( p4 )
#    
#     create v3
    v3=hm.GenVertex()
    evt.add_vertex( v3 )
    v3.add_particle_in( p3 )
    v3.add_particle_in( p4 )
    p6=hm.GenParticle( hm.FourVector(-3.813,0.113,-1.833,4.233 ),22, 1 )
    evt.add_particle( p6 )
    p6.add_attribute("flow1", hm.IntAttribute(231))
    p6.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p6.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v3.add_particle_out( p6 )
    p5=hm.GenParticle(hm.FourVector(1.517,-20.68,-20.605,85.925),-24, 3 )
    evt.add_particle( p5 )
    p5.add_attribute("flow1", hm.IntAttribute(243))
    p5.add_attribute("theta", hm.DoubleAttribute(random.random()*math.pi))
    p5.add_attribute("phi", hm.DoubleAttribute(random.random()*math.pi*2))
    v3.add_particle_out( p5 )
#    
#     create v4
    v4=hm.GenVertex(hm.FourVector(0.12,-0.3,0.05,0.004))
    evt.add_vertex( v4 )
    v4.add_particle_in( p5 )
    p7=hm.GenParticle(hm.FourVector(-2.445,28.816,6.082,29.552), 1,1 )
    evt.add_particle( p7 )
    v4.add_particle_out( p7 )
    p8=hm.GenParticle(hm.FourVector(3.962,-49.498,-26.687,56.373), -2,1 )
    evt.add_particle( p8 )
    v4.add_particle_out( p8 )
#    
#     tell the event which vertex is the signal process vertex
    evt.add_attribute("signal_process_vertex", hm.IntAttribute(v3.id()))
    
    bwrong1=hm.FourVector(-1.1,-0.3,0.2,0)
#  Test that wrong boost will not work
    print (dir(evt))
    assert(False==evt.boost(bwrong1))    
    bwrong2=hm.FourVector (-1.0,-0.0,0.0,0)
# Test that boost with v=c will not work
    assert(False==evt.boost(bwrong2))
    bwrong3=hm.FourVector(sys.float_info.epsilon*0.9,0.0,0.0,0)
#  Test that boost with v=0 will be OK
    assert(True==evt.boost(bwrong3))
    rz=hm.FourVector(0.0,0.0,-0.9,0)
    rzinv=hm.FourVector(0.0,0.0,0.9,0)
    evt.rotate(rz)
    evt.rotate(rzinv)
    evt.clear()
    return 0
if __name__ == "__main__":
    result=1
    result=test_Boost()
    try:
     result=test_Boost()
    except:
     print ("FAILED")
     result=1
    sys.exit(result)



from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
import random
def conversion_factor( from_, to_ ):
    m=hm.FourVector( 0.5-random.random(), 0.5-random.random(), 0.5-random.random(), 0.5-random.random())
    msave=hm.FourVector(m)
    hm.Units.convert(m,from_,to_ )
    return m.e()/msave.e()

def neq(a,b):
	if abs(a-b)<0.001*(abs(a)+abs(b)): return False
	return True

def test_Units():
    err = 0
    evt=hm.GenEvent()

    print ("Default hm.Units: ", hm.Units.name(evt.momentum_unit())," ", hm.Units.name(evt.length_unit()))
    cf = conversion_factor( hm.Units.MomentumUnit.GEV, hm.Units.MomentumUnit.GEV )
    print (cf)
    if  neq(cf,1.0):

        err=err+1
        print ("wrong conversion factor ",cf," for GEV to GEV - should be 1 \n")
    
    cf =  conversion_factor( hm.Units.MomentumUnit.MEV, hm.Units.MomentumUnit.MEV )
    if  neq(cf,1.0 ):
    
        err=err+1
        print ("wrong conversion factor ",cf," for MEV to MEV - should be 1 \n")
    
    cf =  conversion_factor( hm.Units.MomentumUnit.MEV, hm.Units.MomentumUnit.GEV )
    if  neq(cf,0.001 ):
    
        err=err+1
        print ("wrong conversion factor ",cf," for MEV to GEV - should be 0.001 \n")
    
    cf =  conversion_factor( hm.Units.MomentumUnit.GEV, hm.Units.MomentumUnit.MEV )
    if  neq(cf,1000.0) :
    
        err=err+1
        print ("wrong conversion factor ",cf," for GEV to MEV - should be 1000 \n")
    

    cf =  conversion_factor( hm.Units.LengthUnit.MM, hm.Units.LengthUnit.MM )
    if  neq(cf,1.0 ):
    
        err=err+1
        print ("wrong conversion factor ",cf," for MM to MM - should be 1 \n")
    
    cf =  conversion_factor( hm.Units.LengthUnit.CM, hm.Units.LengthUnit.CM )
    if  neq(cf,1.0 ):
    
        err=err+1
        print ("wrong conversion factor ",cf," for CM to CM - should be 1 \n")
    
    cf =  conversion_factor( hm.Units.LengthUnit.CM, hm.Units.LengthUnit.MM )
    if  neq(cf,10.0 ):
    
        err=err+1
        print ("wrong conversion factor ",cf," for CM to MM - should be 10 \n")
    
    cf =  conversion_factor( hm.Units.LengthUnit.MM, hm.Units.LengthUnit.CM )
    if  neq(cf,0.1 ):
    
        err=err+1
        print ("wrong conversion factor ",cf," for MM to CM - should be 0.1 \n")
    
    print (err)
    assert(0 == err)
    return 0
if __name__ == "__main__":
    result=1
    try:
     result=test_Units()
    except:
     result=1
    sys.exit(result)



from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
import random

def test_Pythonization_FourVector():
    evt=hm.GenEvent()
    m=hm.FourVector( 0.5-random.random(), 0.5-random.random(), 0.5-random.random(), 0.5-random.random())
    print( m[0])
    m[1]=10
    return 0


if __name__ == "__main__":
    result=1
    try:
     result=test_Pythonization_FourVector()
    except:
     result=1
    sys.exit(result)








from pyHepMC3 import HepMC3 as hm
import sys

class Pythia8ToHepMC3:
 def __init__(self):
        self.m_internal_event_number=0
        self.m_print_inconsistency=True
        self.m_free_parton_warnings=True
        self.m_crash_on_problem=False
        self.m_convert_gluon_to_0=False
        self.m_store_pdf=True
        self.m_store_proc=True
        self.m_store_xsec=True
        self.m_store_weights=True

    # The recommended method to convert Pythia events into HepMC ones
 def fill_next_event1(self,pythia,evt,ievnum):
        return self.fill_next_event(pythia.event, evt, ievnum, pythia.info, pythia.settings)


    # Alternative method to convert Pythia events into HepMC ones
 def  fill_next_event(self,pyev,evt,ievnum,pyinfo,pyset):
        # 1. Error if no event passed.
        if (evt is None):
            print( "Pythia8ToHepMC3::fill_next_event error - passed null event.")
            return False
        # Event number counter.
        if ( ievnum >= 0 ):
            evt.set_event_number(ievnum)
            self.m_internal_event_number = ievnum
        else:
            evt.set_event_number(self.m_internal_event_number)
            self.m_internal_event_number=self.m_internal_event_number+1
        evt.set_units(hm.Units.GEV,hm.Units.MM)
#        // 2. Fill particle information
        hepevt_particles = []
        for  i in range(0, pyev.size()):
            hepevt_particles.append( hm.GenParticle( hm.FourVector( pyev[i].px(), pyev[i].py(), pyev[i].pz(), pyev[i].e() ),pyev[i].id(), pyev[i].statusHepMC() ))
            hepevt_particles[i].set_generated_mass( pyev[i].m() )
#        // 3. Fill vertex information and find beam particles.
        #For type compatibility
        vertex_cache=    hm.GenEvent().vertices()
        beam_particles = hm.GenEvent().particles()
        for i in range(0, pyev.size()):
            mothers = pyev[i].motherList()
            if  len(mothers) != 0 :
                prod_vtx = hepevt_particles[mothers[0]].end_vertex()
                if prod_vtx is None :
                    prod_vtx = hm.GenVertex()
                    vertex_cache.append(prod_vtx)
                    for j in range (0, len(mothers)):
                        prod_vtx.add_particle_in( hepevt_particles[mothers[j]])
                prod_pos=hm.FourVector ( pyev[i].xProd(), pyev[i].yProd(),pyev[i].zProd(), pyev[i].tProd() )
#                // Update vertex position if necessary
                if ((not prod_pos.is_zero()) and  prod_vtx.position().is_zero()):
                   prod_vtx.set_position( prod_pos )
                prod_vtx.add_particle_out( hepevt_particles[i] )
            else:
               beam_particles.append(hepevt_particles[i])
#        // Add particles and vertices in topological order
        if (len(beam_particles)<2):
         print("There are  ", len(beam_particles), "!=2 particles without mothers")
         if (self.m_crash_on_problem ):
           sys.exit(1)
        evt.add_tree( beam_particles )
#        //Attributes should be set after adding the particles to event
        for i  in range(0, pyev.size()):
#            // Colour flow uses index 1 and 2.
            colType = pyev[i].colType()
            if (colType ==  -1 or colType ==  1 or colType == 2):
                flow1=0
                flow2=0
                if (colType ==  1 or colType == 2):
                  flow1=pyev[i].col()
                if (colType == -1 or colType == 2):
                  flow2=pyev[i].acol()
                hepevt_particles[i].add_attribute("flow1",hm.IntAttribute(flow1))
                hepevt_particles[i].add_attribute("flow2",hm.IntAttribute(flow2))
#        // If hadronization switched on then no final coloured particles.
        if (pyset==None):
          doHadr = self.m_free_parton_warnings and pyset.flag("HadronLevel:Hadronize")
        else:
         doHadr = pyset.flag("HadronLevel:all") and pyset.flag("HadronLevel:Hadronize")

#        // 4. Check for particles which come from nowhere, i.e. are without
#        // mothers or daughters. These need to be attached to a vertex, or else
#        // they will never become part of the event.
        for i in range (1,pyev.size()):

#            // Check for particles not added to the event
#            // NOTE: We have to check if this step makes any sense in HepMC event standard
            if ( not hepevt_particles[i] ):
                print ("hanging particle " , i)
                prod_vtx=hm.GenVertex()
                prod_vtx.add_particle_out( hepevt_particles[i] )
                evt.add_vertex(prod_vtx)

#            // Also check for free partons (= gluons and quarks; not diquarks?).
            if ( doHadr and self.m_free_parton_warnings ):
                if ( hepevt_particles[i].pid() == 21 and  (  hepevt_particles[i].end_vertex() is None ) ):
                    print( "gluon without end vertex ", i)
                    if ( self.m_crash_on_problem ):
                     sys.exit(1)
                if ( abs(hepevt_particles[i].pid()) <= 6 and ( hepevt_particles[i].end_vertex() is None) ):
                    print( "quark without end vertex " , i )
                    if ( self.m_crash_on_problem ):
                     sys.exit(1)

#        // 5. Store PDF, weight, cross section and other event information.
#        // Flavours of incoming partons.
        if (self.m_store_pdf and  pyinfo is not None):
            id1pdf = pyinfo.id1pdf()
            id2pdf = pyinfo.id2pdf()
            if ( self.m_convert_gluon_to_0 ):
                if (id1pdf == 21): id1pdf = 0
                if (id2pdf == 21): id2pdf = 0
            pdfinfo =hm.GenPdfInfo()
            pdfinfo.set(id1pdf, id2pdf, pyinfo.x1pdf(), pyinfo.x2pdf(), pyinfo.QFac(), pyinfo.pdf1(), pyinfo.pdf2() )
#            // Store PDF information.
            evt.set_pdf_info( pdfinfo );
            
#        // Store process code, scale, alpha_em, alpha_s.
        if (self.m_store_proc and pyinfo is not None):
            evt.add_attribute("mpi",hm.IntAttribute(pyinfo.nMPI()))
            evt.add_attribute("signal_process_id",hm.IntAttribute( pyinfo.code()))
            evt.add_attribute("event_scale",hm.DoubleAttribute(pyinfo.QRen()))
            evt.add_attribute("alphaQCD",hm.DoubleAttribute(pyinfo.alphaS()))
            evt.add_attribute("alphaQED",hm.DoubleAttribute(pyinfo.alphaEM()))

#        // Store cross-section information in pb.
        if (self.m_store_xsec and pyinfo is not None):
            xsec = hm.GenCrossSection()
            xsec.set_cross_section( pyinfo.sigmaGen() * 1e9, pyinfo.sigmaErr() * 1e9)
            evt.set_cross_section(xsec)

#        // Store event weights.
        if (self.m_store_weights and pyinfo is not None):
            evt.weights().clear()
            for iweight in range(0,pyinfo.nWeights()):
                evt.weights().append(pyinfo.weight(iweight))
 
#        // Done.
        return True



import re
import sys,os

def update_path():
 v=sys.version_info
 return [os.path.abspath(os.path.join(os.pardir,str(v[0])+"."+str(v[1])+"."+str(v[2])))]+[os.getcwd()]+sys.path
  
def COMPARE_ASCII_FILES(f1,f2):
 file1=open(f1)
 file2=open(f2)
 print ("Run comparison")
 string1=" "
 string2=" "
 j=0
 while string1 and string2:
     j=j+1
     string1=file1.readline()
     string2=file2.readline()
     if (string1 != string2):
            print (j,"-th strings are not equal", "\n")
            print ("   ", string1 ,"\n")
            print ("   ", string2 ,"\n")
            if not re.match(r'HepMC::Version',string1):
             file1.close()
             file2.close()
             return 1
 file1.close()
 file2.close()
 return 0



from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
import random

def test_Pythonization_GenRunInfo():
    ri = hm.GenRunInfo()
    #ri.Tools = [("foo", "1.0", "bar")]
    a=hm.GenRunInfo.ToolInfo()
    a.name="aaa"
    b=hm.GenRunInfo.ToolInfo()
    c=hm.GenRunInfo.ToolInfo()
    b.name="foo"
    c= ("foo", "1.0", "bar")
    #ri.tools().append(a)
    #ri.tools().append(b)
    #print (ri.tools())
    #print (len(ri.tools()))  
    #print (a.name  )
    #print (ri.tools()[1].name )
    #ri.tools()[1]=a
    #ri.tools()=[("foo", "1.0", "bar"),("foo2", "12.0", "bar2")]
    #ri.set_weight_names(["a", "b", "c"])
    #ri.Weight_names = ["a", "b", "c"]
    return 0

if __name__ == "__main__":
    result=1
    try:
     result=test_Pythonization_GenRunInfo()
    except:
     result=1
    sys.exit(result)








from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from pyHepMC3 import HepMC3 as hm
import pythia8 as p8
from Pythia8ToHepMC3 import Pythia8ToHepMC3 
def test_Pythia8():
 try:
  pythia = p8.Pythia()
 except ImportError:
  return 0
 pythia.readString("WeakSingleBoson:ffbar2gmZ = on")
 pythia.readString("Beams:idA =  11")
 pythia.readString("Beams:idB = -11")
 pythia.readString("Beams:eCM =  91.2")
 pythia.init()
 p8tohm=Pythia8ToHepMC3 ()
 p8tohm.m_store_pdf=True
 out=hm.WriterAscii("test_Pythia.hepmc")
 for iEvent in range(0, 100):
    if not pythia.next(): continue
    nCharged = 0
    evt=hm.GenEvent()
    p8tohm.fill_next_event1(pythia,evt,iEvent)
    out.write_event(evt)
    evt.clear()
 pythia.stat()
 out.close()
 return 0
if __name__ == "__main__":
    result=1
    try:
     result=test_Pythia8()
    except:
     result=1
    sys.exit(result)



from  pyHepMC3TestUtils import update_path
import sys,os
sys.path=update_path()

from  pyHepMC3TestUtils import COMPARE_ASCII_FILES
from pyHepMC3 import HepMC3 as hm
import random

def test_Pythonization_docs():
    print (hm.GenEvent.particles.__doc__)
    print (hm.GenEvent.event_pos.__doc__)
    print (hm.GenVertex.particles_in.__doc__)
    return 0    

if __name__ == "__main__":
    result=1
    try:
     result=test_Pythonization_docs()
    except:
     result=1
    sys.exit(result)








import os
import setuptools
import sys
import platform, shutil
from setuptools import find_packages, setup, Extension
from setuptools.command.build_ext import build_ext as build_ext_orig
from setuptools.command.install_lib import install_lib as install_lib_orig
from subprocess import check_output
from shutil import copyfile
from distutils.sysconfig import get_python_lib;
#https://stackoverflow.com/questions/42585210/extending-setuptools-extension-to-use-cmake-in-setup-py
class CMakeExtension(Extension):
    def __init__(self, name, sources=[]):
        Extension.__init__(self,name = name, sources = sources)

def get_hepmc3_version():
  line = '#define HEPMC3_VERSION_CODE 3002000'
  current=os.getcwd()
  with open(current+'/include/HepMC3/Version.h') as f:
    line = next((l for l in f if 'HEPMC3_VERSION_CODE' in l and '#define ' in l), None)
  number=int(line.split(' ')[2]) 
  return str(int(number/1000000))+'.'+str(int((number%1000000)/1000))+'.'+str((number%1000))

def get_library_location():
 ps=platform.system()
 bits=platform.architecture()[0]
 if  ps == 'Solaris':
  return "lib"
 if  ps == 'FreeBSD':
  return "lib"
 if  ps == 'Linux':
  if bits=='64bit':
   return "lib64"
  if bits=='32bit':
   return "lib"
 if  ps == 'Darwin':
  return "lib"
 if  ps == 'Windows':
  return "lib"
 return "lib"
 
def get_hepmc3_libraries():
 lib=get_library_location()
 ps=platform.system()
 if  ps == 'Darwin':
  return [( lib, ['outputs/'+lib+'/libHepMC3.dylib',  'outputs/'+lib+'/libHepMC3search.dylib','outputs/'+lib+'/libHepMC3-static.a','outputs/'+lib+'/libHepMC3search-static.a'])]
 if  ps == 'Windows':
  return [( lib, ['outputs/lib/HepMC3.dll', 'outputs/'+lib+'/HepMC3search.dll', 'outputs/'+lib+'/HepMC3search-static.lib', 'outputs/'+lib+'/HepMC3-static.lib'])]
 return [( lib, ['outputs/'+lib+'/libHepMC3.so', 'outputs/'+lib+'/libHepMC3.so.3','outputs/'+lib+'/libHepMC3search.so','outputs/'+lib+'/libHepMC3search.so.3'])]

class  hm3_install_lib(install_lib_orig):
      def run(self):
        cwd=os.path.abspath(os.getcwd())
        v=sys.version_info
        versionstring=str(v[0])+"."+str(v[1])+"."+str(v[2])
        shutil.copytree(os.path.join(cwd,"python",versionstring,"pyHepMC3"),os.path.join(cwd,self.build_dir,"pyHepMC3"))
        print(install_lib_orig.get_outputs(self)) 
        print(self.install_dir) 
        print(self.build_dir) 
        install_lib_orig.run(self)

class hm3_build_ext(build_ext_orig):
    def get_ctest_exe(self):
      return "ctest"
    def get_cmake_exe(self):
        vg='0'
        cmakeg_exe=""
        outg = check_output(["cmake", "--version"])
        outgsplit=outg.split()
        if len(outgsplit)>2: 
           vg=outgsplit[2]
           if int(vg[0])>=3: 
            cmakeg_exe="cmake"
        if (cmakeg_exe!=""): return cmakeg_exe

        v3='0'
        cmake3_exe=""
        out3 = check_output(["cmake3", "--version"])
        out3split=out3.split()
        if len(out3split)>2:
         v3=out3split[2]
         if int(v3[0])==3: 
           cmake3_exe="cmake3"
        if (cmake3_exe!=""): return cmake3_exe
        return "foo "
    def get_cmake_python_flags(self):
        pv=sys.version_info
        return '-DHEPMC3_PYTHON_VERSIONS='+str(pv[0])+'.'+str(pv[1])

        
    def run(self):
        for ext in self.extensions:
            self.build_cmake(ext)
#        build_ext_orig.run(self)

    def build_cmake(self, ext):
        build_temp=os.getcwd()
        cwd=os.path.abspath(os.getcwd())
        cmake_exe=self.get_cmake_exe()
        ctest_exe=self.get_ctest_exe()
        cmake_args = [
         'CMakeLists.txt',
         '-DHEPMC3_BUILD_EXAMPLES:BOOL=OFF',
         '-DHEPMC3_INSTALL_INTERFACES:BOOL=OFF',
         '-DHEPMC3_ENABLE_SEARCH:BOOL=ON',
         '-DHEPMC3_BUILD_DOCS:BOOL=OFF',
         '-DHEPMC3_ENABLE_PYTHON:BOOL=ON',
         '-DHEPMC3_ENABLE_ROOTIO:BOOL=OFF',
         '-DCMAKE_BUILD_TYPE=Release', 
         '-DHEPMC3_ENABLE_TEST:BOOL=ON',
         self.get_cmake_python_flags()]
        ps=platform.system()
        bits=platform.architecture()[0]
        if  ps == 'Linux':
          if bits=='64bit':
            cmake_args.append ('-DLIB_SUFFIX=64')
            cmake_args.append ('-DCMAKE_INSTALL_LIBDIR=lib64')
        if ps == 'Windows':
          #FIXME: detect arch
          cmake_args.append ('-Thost=x64')
          cmake_args.append ('-A')
          cmake_args.append ('x64')
        self.spawn([cmake_exe, str(cwd)] + cmake_args)

        if not self.dry_run:
            build_args = [  ]
            self.spawn([cmake_exe, '--build', '.'] + build_args)
            ctest_args = []
            v=sys.version_info
            if  ps == 'Windows':
             ctest_args.append('-C')
             ctest_args.append('Debug')
            #Travis Windows bug?
            if ps != 'Darwin' and  not ( ps == 'Windows' and v[0]==3 and v[1] == 8 ):
             self.spawn([ctest_exe,  '.','--output-on-failure']+ctest_args)
        os.chdir(str(cwd))
def local_find_packages():
  os.mkdir('pyHepMC3')
  return ['pyHepMC3']


setuptools.setup(
     name='HepMC3',  
     version=get_hepmc3_version(),
     author="HepMC3 Developers",
     author_email="hepmc-dev@cern.ch",
     description="HepMC3 library and Python bindings for HepMC3",
     long_description="Official python bindings for the HepMC3 library. Please visit https://hepmc.web.cern.ch/hepmc/ and  https://gitlab.cern.ch/hepmc/HepMC3 for more documentation",
     long_description_content_type="text/markdown",
     url="https://gitlab.cern.ch/hepmc/HepMC3",
     license = "GPLv3",
     platforms=['any'],
     include_package_data = True,
     packages=local_find_packages(),
     data_files=get_hepmc3_libraries()+[
     ('bin',                ['outputs/bin/HepMC3-config']),       
     ('share/HepMC3/cmake', [
     'outputs/share/HepMC3/cmake/HepMC3Config-version.cmake',
     'outputs/share/HepMC3/cmake/HepMC3Config.cmake']),       
     ('include/HepMC3',[
     'include/HepMC3/ReaderPlugin.h','include/HepMC3/WriterPlugin.h',
     'include/HepMC3/WriterAsciiHepMC2.h','include/HepMC3/WriterHEPEVT.h','include/HepMC3/Units.h',
     'include/HepMC3/HEPEVT_Wrapper.h','include/HepMC3/GenCrossSection.h','include/HepMC3/GenRunInfo.h',
     'include/HepMC3/WriterAscii.h','include/HepMC3/Setup.h',
     'include/HepMC3/GenVertex.h','include/HepMC3/FourVector.h','include/HepMC3/PrintStreams.h',
     'include/HepMC3/GenEvent.h','include/HepMC3/ReaderHEPEVT.h','include/HepMC3/Print.h',
     'include/HepMC3/LHEF.h','include/HepMC3/GenParticle_fwd.h',
     'include/HepMC3/Reader.h','include/HepMC3/GenPdfInfo_fwd.h','include/HepMC3/GenParticle.h',
     'include/HepMC3/GenCrossSection_fwd.h','include/HepMC3/LHEFAttributes.h',
     'include/HepMC3/AssociatedParticle.h','include/HepMC3/ReaderLHEF.h','include/HepMC3/GenPdfInfo.h',
     'include/HepMC3/HepMC3.h','include/HepMC3/Errors.h',
     'include/HepMC3/GenHeavyIon.h','include/HepMC3/Writer.h','include/HepMC3/ReaderFactory.h',
     'include/HepMC3/ReaderAsciiHepMC2.h',
     'include/HepMC3/Version.h','include/HepMC3/Attribute.h','include/HepMC3/GenHeavyIon_fwd.h',
     'include/HepMC3/ReaderAscii.h',
     'include/HepMC3/GenVertex_fwd.h','search/include/HepMC3/Filter.h',
     'search/include/HepMC3/Relatives.h', 'search/include/HepMC3/AttributeFeature.h',
     'search/include/HepMC3/FilterAttribute.h', 'search/include/HepMC3/Feature.h',
     'search/include/HepMC3/Selector.h'
      ]),
     ('include/HepMC3/Data',[
     'include/HepMC3/Data/GenEventData.h',
     'include/HepMC3/Data/GenParticleData.h',
     'include/HepMC3/Data/GenVertexData.h',
     'include/HepMC3/Data/GenRunInfoData.h'])
                  ],
     classifiers=[
         "Programming Language :: Python ",
         "Operating System :: OS Independent",
     ],
     ext_modules=[CMakeExtension('pyHepMC3')],
      cmdclass={
        'build_ext': hm3_build_ext,  'install_lib': hm3_install_lib
    }
 )
